/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Descriptions    : Combines all levels from DFR into 1 cohesive file by MLT
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:

    Parameters
        vTabName    : Table being truncated
        vObjectType : T = Table, S = Subpartition and P = Partition
        vObjectName : Partition or Subpartition name
        OraPath     : Oracle SID
        OraUser     : Oracle User ID
        OraPW       : Oracle Password

    Process Flow
        1) Get lowest level of Merch and Loc from the HIER_LEVEL table in the RDBMS
        2) Adjust Location level and ID if cluster or cluster sets
        3) Attach Time IDs
        4) Transpose data based on a column containing KPI names as the target column names
           and a column called PREDICT to be used as the columns target values
        5) Call macro to rename column names from standard to actual KPI names from MP system
-------------------------------------------------------------------------------------------------------*/


%macro dfr_import_add_parents();
    proc sql noprint;

	    /* Merchandise Parents */
        %if &merch_from_level.<&merch_to_level. %then 
            %do;
                create table mkeys as 
                select * from stage.mkeys_&projid.;

                %do m=&merch_from_level. %to %eval(&merch_to_level.-1);
                    %if &m<=&dfr_maxmerch or &m=&merch_to_level. 
                        %then 
                            %do;
                                %if &m. = 1 %then 
                                    %do; 
                                        %let merchtab = cmast; 
                                    %end;
                                %else 
                                    %do; 
                                        %let merchtab = ctree; 
                                    %end;

                                insert into stage.mkeys_&projid. (merch_id, merch_level) 
                                select distinct(lv&m.&merchtab._id), &m. from maxdata.lv&merch_to_level.ctree
                                where lv&merch_to_level.ctree_id in (select merch_id from mkeys);
                            %end;
                %end;
            %end;

    	/* Static Location parents */
        %if (&orig_loc_from_level.<1000) and (&loc_from_level.<&l_lev.) %then 
            %do; /* static location hierarchy */
                create table lkeys as select * from stage.lkeys_&projid.;

                %do l=&loc_from_level. %to %eval(&l_lev.-1);
                    insert into stage.lkeys_&projid. (location_id, location_level) 
                    select distinct (lv&l.loc_id), &l. from maxdata.lv&loc_to_level.loc 
                    where lv&loc_to_level.loc_id in (select location_id from lkeys);
                %end;
            %end;

	/* Time parents 
    %if &time_from_level.<&time_to_level. %then %do;
        create table tkeys as select * from stage.tkeys_&projid.;
        %do t=&time_from_level. %to %eval(&time_to_level.-1);
            insert into stage.tkeys_&projid. (time_id, time_level)
                select distinct(lv&t.time_lkup_id),%eval(&t.+46) from maxapp.lv&time_to_level.time 
                    where lv&time_to_level.time_lkup_id in (select time_id from tkeys);
            %end;
        %end;
    */
    /*
    %if (&time_template.>0) %then %do; 
        insert into stage.tkeys_&projid. (time_id, time_level) values (&from_time_id., 45);
        %end;
    */

    quit;
%mend;



%macro Process_level(DSN=,Time_to_Level=,MERCH_FROM_LEVEL=,MERCH_TO_LEVEL=,loc_from_level=,loc_to_Level=,orig_loc_from_level=,dfr_clstr_st_id=,FV_Version_ID=,adjustedmlevel=);
    /*
        Compute the highest merch/loc levels as a reletive number
        Add 1 to the max to establish the highest level relative to level 10 (SKU)
        Doing tyhis is so we can have a linear count of levels and account for skipped
        levels
    */
    proc sql noprint;
        select max(level_id-10) as MAX_MERCH_LEVEL
             , max(level_id-10+1) as NEXT_MERCH_LEVEL
          into :MAX_MERCH_LEVEL
             , :NEXT_MERCH_LEVEL
          from maxdata.hier_level
         where Hier_ID = 11 and Level_ID < 20
        ;
    quit;

    proc sql noprint;
        select max(level_id) as MAX_LOC_LEVEL
             , max(level_id+1) as NEXT_LOC_LEVEL
          into :MAX_LOC_LEVEL
             , :NEXT_LOC_LEVEL
          from maxdata.hier_level
         where Hier_ID = 1 and Level_ID < 4
        ;
    quit;

    /*  This removes leading and trailing blanks from macro variables calculated above in SQL*/
    %let MAX_MERCH_LEVEL=&MAX_MERCH_LEVEL.;
    %let NEXT_MERCH_LEVEL=&NEXT_MERCH_LEVEL.;

    /*  Get the current merch and loc levels from the dataset name
    */
    %let Merch_Level=&merch_to_level.;
    %let Location_Level=&loc_to_level.;

    %IF (&Merch_Level. > &MAX_MERCH_LEVEL.) %Then
        %do;
            /*  Since there is no real level after the max level, we reset the level to 10(SKU)*/
            %let Current_Merch_Level=10;
        %end;
    %else
        %do;
            %let Current_Merch_Level=&Merch_Level.;
        %end;

    /*  PADETT: No idea what this is for. Need to ask Dale Matheny or RIS - Pune, India*/
    %let incl_reconstat=1;
    %if (&adjustedmlevel.=&recon_merch.) or (&dfr_wf_recon.=0) %then %let incl_reconstat=0;

    /*
        Cluster Adjustments

    */
    %If (&orig_loc_from_level. > 1000) %Then
        %Do;
            /* Process Levels as Clusters*/
            %Let Orig_Loc_Level=&Location_Level.;

            %If (&Orig_Loc_Level.=1) %Then
                %Do;
                    %If (&orig_loc_from_level.=1001) %Then
                        %Do;
                            %Let Location_Level=1001;
                            %Let Location_ID=&dfr_clstr_st_id.;
                        %End;
                    %else
                        %Do;
                            %PUT ERROR: Location Level 1 Cluster Set Violation;
                            %*Let Location_Level=1002;
                            %*Let Location_ID=&orig_loc_from_id.;
                        %End;
                %End;

            %If (&Orig_Loc_Level.=2) %Then
                %Do;
                    %If (&orig_loc_from_level.=1002) %Then
                        %Do;
                            %Let Location_Level=1002;
                            %Let Location_ID=&orig_loc_from_id.;
                        %End;
                    %else
                        %Do;
                            %Let Location_ID=LID_&Orig_Loc_Level.;
                            %Let Location_Level=1002;
                        %End;
                %End;

            %If (&Orig_Loc_Level.>2) %Then
                %Do;
                    %Let Location_ID=LID_&Location_Level.;
                    %Let Location_Level=4;
                %End;
        %End;
    %Else
        %Do;
            %Let Location_ID=LID_&Location_Level.;
            %if (&Location_Level.>&MAX_LOC_LEVEL.) %then %Let Location_Level=4;
        %End;

    /*  Expand out merch and loc data by time intervals
        Because the time tables are by week, need to compare date ranges
        PADETT: Need to describe below
            %if ((&incl_reconstat.>0) and (&recon_single.=1)) %then %do; and _reconstatus_ in (0,10000) %end;
           and actual is null

        PADTT: 14APR2009 - SIGNIFIGANT CHANGES
        Modified compar logic to compare MMYYYY as charater instead of dated

    */

    /*
    proc sql;
        create table out.&DSN._Temp001 as
        select &Current_Merch_Level. as Merch_Level
             , MID_&Merch_Level as Merch_ID
             , &Location_Level as Location_Level
             , &Location_ID. as Location_ID
             , (&Time_to_Level. + 46) as Time_Level
             , time_&Time_to_Level.
             , put(time_&Time_to_Level.,monyy7.) time_check format=$7
             , 0 as Time_ID
             , _NAME_
             , PREDICT label='PREDICT'
             , STD label='STD'
          from out.&DSN.
          where time_&time_to_level.>= "&from_time."d and time_&time_to_level.<= "&to_time."d
            %if ((&incl_reconstat.>0) and (&recon_single.=1)) %then %do; and _reconstatus_ in (0,10000) %end;
           and (actual is null or actual=0)
        ;
    quit;
    */


    /*
        padett: 21APR2009 - Need to add conversions for columns time_x 
                            Original assumption was that all time_x columns where monyy7.
                            During testing of qtrly wrksht time_3 was YYQ6.
    */
    data out.&DSN.;
        set out.&DSN.;
        format time_to_monyy7 MONYY7.;
        time_to_monyy7 = time_&Time_to_Level.;
    run;

    /*
        padett: 21APR2009 - Added sepearte routines to handle week, month and qtr
    */

    %if &time_to_level. eq 5 %then
        %Do;
            proc sql;
                create table out.&DSN._Temp001 as
                select &Current_Merch_Level. as Merch_Level
                     , MID_&Merch_Level as Merch_ID
                     , &Location_Level as Location_Level
                     , &Location_ID. as Location_ID
                     , (&Time_to_Level. + 46) as Time_Level
                     , time_&Time_to_Level.
                     , put(time_to_monyy7,monyy7.) as time_check format=$7.
                     , 0 as Time_ID
                     , _NAME_
                     , PREDICT label='PREDICT'
                     , STD label='STD'
                  from out.&DSN.
                 where intnx('week',time_&time_to_level.,0,'end') >= "&from_time."d
                   and intnx('week',time_&time_to_level.,0,'beginning') <= "&to_time."d
                   %if ((&incl_reconstat.>0) and (&recon_single.=1)) %then %do; and _reconstatus_ in (0,10000) %end;
                   and (actual is null or actual=0)
                ;
            quit;

            proc sql;
                update out.&DSN._Temp001
                   set time_id = (select distinct LV&Time_to_Level.TIME_LKUP_ID
                                    from inp.lv&Time_to_Level.time_&projid.
                                   where time_&Time_to_Level. >= datepart(LV&Time_to_Level.TIME_START_DATE)
                                     and time_&Time_to_Level. <= datepart(LV&Time_to_Level.TIME_END_DATE))
                ;
            quit;
        %End;
    /*
        padett: 21APR2009 - Added sepearte routines to handle week, month and qtr
    */

    %if &time_to_level. eq 4 %then
        %Do;
            proc sql;
                create table out.&DSN._Temp001 as
                select &Current_Merch_Level. as Merch_Level
                     , MID_&Merch_Level as Merch_ID
                     , &Location_Level as Location_Level
                     , &Location_ID. as Location_ID
                     , (&Time_to_Level. + 46) as Time_Level
                     , time_&Time_to_Level.
                     , put(time_to_monyy7,monyy7.) as time_check format=$7.
                     , 0 as Time_ID
                     , _NAME_
                     , PREDICT label='PREDICT'
                     , STD label='STD'
                  from out.&DSN.
                 where intnx('month',time_&time_to_level.,0,'end') >= "&from_time."d
                   and intnx('month',time_&time_to_level.,0,'beginning') <= "&to_time."d
                   %if ((&incl_reconstat.>0) and (&recon_single.=1)) %then %do; and _reconstatus_ in (0,10000) %end;
                   and (actual is null or actual=0)
                ;
            quit;

            %let date_adjust=10;
            data inp.lv&Time_to_Level.time_&projid._Modified;
                set inp.lv&Time_to_Level.time_&projid.;
                format LV&Time_to_Level.TIME_LKUP_ID_MONYYYY $7.;
                LV&Time_to_Level.TIME_LKUP_ID_MONYYYY=put(intnx('day',datepart(LV&Time_to_Level.TIME_START_DATE),&date_adjust.),monyy7.);
            run;

            proc sql;
                update out.&DSN._Temp001
                    set time_id = (select LV&Time_to_Level.TIME_LKUP_ID
                                     from inp.lv&Time_to_Level.time_&projid._Modified
                                    where upcase(time_check) = LV&Time_to_Level.TIME_LKUP_ID_MONYYYY)
                ;
            quit;

        %End;
    /*
        padett: 21APR2009 - Added sepearte routines to handle week, month and qtr
    */
    %if &time_to_level. eq 3 %then
        %Do;
            proc sql;
                create table out.&DSN._Temp001 as
                select &Current_Merch_Level. as Merch_Level
                     , MID_&Merch_Level as Merch_ID
                     , &Location_Level as Location_Level
                     , &Location_ID. as Location_ID
                     , (&Time_to_Level. + 46) as Time_Level
                     , time_&Time_to_Level.
                     , put(time_&Time_to_Level.,YYQC6.) as time_check format=$6.
                     , 0 as Time_ID
                     , _NAME_
                     , PREDICT label='PREDICT'
                     , STD label='STD'
                  from out.&DSN.
                 where intnx('qtr3.2',time_&time_to_level.,0,'end') >= "&from_time."d
                   and intnx('qtr3.2',time_&time_to_level.,0,'beginning') <= "&to_time."d
                   %if ((&incl_reconstat.>0) and (&recon_single.=1)) %then %do; and _reconstatus_ in (0,10000) %end;
                   and (actual is null or actual=0)
                ;
            quit;

            %let date_adjust=10;
            data inp.lv&Time_to_Level.time_&projid._Modified;
                set inp.lv&Time_to_Level.time_&projid.;
                format LV&Time_to_Level.TIME_LKUP_ID_QTR $6.;
                LV&Time_to_Level.TIME_LKUP_ID_QTR=put(intnx('day',datepart(LV&Time_to_Level.TIME_START_DATE),&date_adjust.),YYQC6.);
            run;

            proc sql;
                update out.&DSN._Temp001
                    set time_id = (select LV&Time_to_Level.TIME_LKUP_ID
                                     from inp.lv&Time_to_Level.time_&projid._Modified
                                    where upcase(time_check) = LV&Time_to_Level.TIME_LKUP_ID_QTR)
                ;
            quit;

        %End;

/*
                                            where upcase(put(time_&Time_to_Level.,monyy7.)) = LV&Time_to_Level.TIME_LKUP_ID_MONYYYY)
*/

   /*  We may be able to eliminate this sort. For some reason the transpose kept erroring out without it
    */
    proc sort data=out.&DSN._Temp001
               nodupkey;
        by _name_ merch_level merch_id location_level location_id time_level time_id;
    run;

    /*  Need to take the kpi _NAME_ column and split it into multiple columns based on the
        value of _NAME_ as the actual column name having a value of column PREDICT
    */
    proc transpose data= out.&DSN._Temp001
                   out=out.&DSN._trans(where=(merch_level ne . and location_level ne .));
        by _name_ merch_level merch_id location_level location_id time_level time_id;
        id _name_;
        var predict std;
    run;

    /*
        padett: 13MAR2009 - Need to account for Standard Deviation values on Revisit logic
    */

    %If (&Dfr_wf_revisit. eq 0) %Then
        %Do;
	      data out.&DSN._trans(drop=_label_) ;
                set out.&DSN._trans;
                if (_label_ eq 'PREDICT') then output;
              run;
            /*  Macro below renames the columns into valid KPI names*/
            %BuildKPINameset(library_name=out,
                             Dataset_Name=&DSN._trans,
                             Original_Names=&dfr_kpinameset.,
                             New_Names=%str(&DFR_FCAST_IMPORT_KPIS.),
                             FV_Version_ID=&dfr_import_version.);

        %End;
    %Else
        %Do;

            data out.&DSN._Predict(drop=_label_)
                 out.&DSN._Std(drop=_label_);
                set out.&DSN._trans;
                if (_label_ eq 'STD') then
                    do;
                        output out.&DSN._Std;
                    end;
                else
                    do;
                        output out.&DSN._Predict;
                    end;

            run;

            /*  Macro below renames the columns into valid KPI names*/
            %BuildKPINameset(library_name=out,
                             Dataset_Name=&DSN._Predict,
                             Original_Names=&dfr_kpinameset.,
                             New_Names=%str(&DFR_FCAST_IMPORT_KPIS.),
                             FV_Version_ID=&dfr_import_version.);

            /*  Macro below renames the columns into valid KPI names*/
            %BuildKPINameset(library_name=out,
                             Dataset_Name=&DSN._Std,
                             Original_Names=&dfr_kpinameset.,
                             New_Names=%str(&DFR_FCAST_IMPORT_KPIS_STD.),
                             FV_Version_ID=&dfr_import_version.);

            data out.&DSN._trans_1;
                merge out.&DSN._Predict_1 (in=a)
                      out.&DSN._Std_1     (in=b);
                by Merch_Level Merch_ID location_level location_id time_level time_id;
            run;


        %End;
%MEND;

/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:

    Descriptions    : Create table used to house final results
    Parameters:
        Table_Type  : Defines whether to process as a fact table or forecast table
        vFactTabName: The target fact/fcast table name. Used as a definition template to build the temp
                      table for aggregation processing.
        vNewTabName : The temp table with partitions & subpartitions for future processing
        iStartYear  : Beginning year use to get the whole potential range of values
        iEndYear    : Ending year use to get the whole potential range of values
        iFvVersionId: Forecast Version - Note, not needed for standard fact tables

    Process Flow:   1) Execute Create table for either standard Fact Table for Forecast Fact Table
                    2) Generate and execute alter statements to define full range of partitions or
                       subpartitions.
-------------------------------------------------------------------------------------------------------*/
%Macro p_cr_final_table(Table_Type=,
                        vFactTabName=,
                        vNewTabName=,
                        iStartYear=,
                        iEndYear=,
                        iFvVersionId=);

    proc datasets library=maxdata;
        delete &vNewTabName.;
    quit;

    /*  For Standard Fact Tables define table with Partitions. Fore Forecast tables define as Subpartitions */
    %If (%sysfunc(kupcase(&Table_Type NE F))) %then
        %Do;
            PROC SQL;
                CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                EXECUTE (CREATE TABLE &vNewTabName PARTITION BY RANGE (TIME_LEVEL, TIME_ID)
                                (PARTITION MFVER_47_DEFAULT VALUES LESS THAN (47, 1))
                                NOLOGGING NOCOMPRESS AS
                                SELECT * FROM MAXDATA.&vFactTabName WHERE 1 = 2
                        ) BY &dbtype.;
                DISCONNECT FROM &dbtype.;
            QUIT;
        %End;
    %Else
        %Do;
            PROC SQL;
                CONNECT TO &dbtype. (USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.);
                EXECUTE (%str(CREATE TABLE &vNewTabName PARTITION BY RANGE (TIME_LEVEL, TIME_ID)
                              SUBPARTITION BY LIST (FV_VERSION_ID)
                              (PARTITION MFVER_47_DEFAULT VALUES LESS THAN (47, 1) NOLOGGING NOCOMPRESS
                              (SUBPARTITION MFVER_47_DEFAULT_&iFvVersionId VALUES (1) ))
                              AS SELECT * FROM maxdata.&vFactTabName WHERE 1 = 2)
                        ) BY &dbtype.;
                DISCONNECT FROM &dbtype.;
            QUIT;
        %End;

    %RDBMS_Report(macro_call=DFR_IMPORT_P_CR_FINAL_TABLE,code_area=1B,switch=ON);

    /*  Build a list of all potential partitions */
    proc sql noprint;
        create table Temp_Time_Table as
        SELECT '47' AS time_level, lv1time_lkup_id AS time_id
          FROM maxapp.lv1time
         WHERE cycle_id BETWEEN &iStartYear AND &iEndYear
     UNION ALL
        select '48' as time_level, lv2time_lkup_id as time_id
          from maxapp.lv2time
         where cycle_id between &iStartYear and &iEndYear
     union all
        select '49' as time_level, lv3time_lkup_id as time_id
          from maxapp.lv3time
         where cycle_id between &iStartYear and &iEndYear
     union all
        select '50' as time_level, lv4time_lkup_id as time_id
          from maxapp.lv4time
         where cycle_id between &iStartYear and &iEndYear
     union all
        select '51' as time_level, lv5time_lkup_id as time_id
          from maxapp.lv5time
         where cycle_id between &iStartYear and &iEndYear
      order by 1, 2
        ;
    quit;

    %let vPartString=MFVER;
    filename tempsql2 TEMP lrecl=1024;
    *filename tempsql2 'd:\PaulsJob\Testsql.sql' lrecl=1024;

    /*  Loop through and create alter statements to add partitions or subpartitions */
    data _null_;
        format sql_line             $1024.;
        format vPartitionName       $80.;
        format vSubPartitionName    $80.;
        file tempsql2;
        set Temp_Time_Table end=eof;

        IF (_N_ eq 1) then
            do;
                put 'PROC SQL;';
                put "   CONNECT TO &dbtype.(USER=MAXDATA ORAPW=&dbpassword. PATH=&DSN.)" ';';
            end;

        vPartitionName    = kcompress("&vPartString."||'_'||time_level||'_'||put(time_id,16.));
        vSubPartitionName = kcompress("&vPartString."||'_'||time_level||'_'||put(time_id,16.)||'_'||"&iFvVersionId.");

        %If (%sysfunc(kupcase(&Table_Type EQ F))) %then
            %Do;
                sql_line = ' Alter Table '||"&vNewTabName"||' add partition '||kcompress(vPartitionName)
                         ||' values less than ('||time_level||', '||kcompress(put(time_id + 1,16.))
                         ||') (subpartition '||kcompress(vSubPartitionName)||' Values ('||"&iFvVersionId."||'))';
            %End;
        %Else
            %Do;
                sql_line = ' Alter Table '||"&vNewTabName"||' add partition '||kcompress(vPartitionName)
                         ||' values less than ('||time_level||', '||kcompress(put(time_id + 1,16.))
                         ||')';
            %End;

        put 'EXECUTE(' sql_line ') BY &dbtype.;';

        IF (eof eq 1) then
            do;
                put 'QUIT;';
            end;
    run;

    %include tempsql2;

%Mend;

/*------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Description: Renames colums and rolls up data. Because this table took 1 column
                 and used it to create many columns, each column will have null
                 values. For example if there is a column _NAME_ with 3 distinct
                 values A, B and C and 9 rows of data. New columns are created
                 and 3 rows would have data for a while 6 would be null. In the end,
                 there should only be 3 rows with no null values for A, B or C
    Developer: Paul Dettorre, X8265
    Created: 11/2008
    Modification Log:
    Parameters
        library_name    : SAS Library of target dataset
        Dataset_Name    : Dataset whose columns are being renamed
        Original_Names  : Space delimited list of KPI's to rename Example; SALES UNITS
        New_Names       : C0mma delimited list of KPI names to use
        FV_Version_ID   : Forecast Version ID

------------------------------------------------------------------------------*/
%macro BuildKPINameset(library_name=,Dataset_Name=, Original_Names=, New_Names=,FV_Version_ID=);
    /* The purpose of this is to rename KPI column names
        Original names are the current column names of the input dataset
        New Names are the kpi names used in the MP system
        To ensure there is no duplication of keys records are grouped and
        ordered by 6 keys
    */
    %local Current_Source Current_Target;
    %let Current_Source=START;
    %let Current_Target=;

    %let loop_ctr=1;
    PROC SQL;
        CREATE TABLE &library_name..&Dataset_Name._1 AS
        SELECT Merch_Level
             , Merch_ID
             , location_level
             , location_id
             , time_level
             , time_id
             , &FV_Version_ID. as FV_Version_ID
            %do %while (&Current_Source ne );
                %let Current_Source=%sysfunc(kscan(&Original_Names.,&loop_ctr.));
                %let Current_Target=%sysfunc(kscan(&New_Names.,&loop_ctr.,%str(',')));
                %let Current_Target=%sysfunc(kscan(&Current_Target.,2));
                %let loop_ctr=%eval(&loop_ctr.+1);
                %if (&Current_Source. ne ) %Then
                    %Do;
                        , MAX(&Current_Source.) as &Current_target.
                    %End;
            %end;
          FROM &library_name..&Dataset_Name.
      GROUP BY Merch_Level, Merch_ID, location_level, location_id, time_level, time_id
      ORDER BY Merch_Level, Merch_ID, location_level, location_id, time_level, time_id
        ;
    QUIT;
%mend;

/*-------------------------------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Descriptions    : Truncates either whole tables or partitions in SQL
    Developer       : Paul Dettorre, X8265
    Created         : 11/2008
    Modification Log:

    Parameters
        vTabName    : Table being truncated
        vObjectType : T = Table, S = Subpartition and P = Partition
        vObjectName : Partition or Subpartition name
        OraPath     : Oracle SID
        OraUser     : Oracle User ID
        OraPW       : Oracle Password
-------------------------------------------------------------------------------------------------------*/
%Macro p_trunc_table(vTabName=,vObjectType=,vObjectName=,OraPath=&DSN.,OraUser=MAXDATA,OraPW=);
    PROC SQL;
        CONNECT TO &dbtype.(USER=&OraUser. ORAPW=&OraPW. PATH=&OraPath.);
        EXECUTE(
                %If (&vObjectType eq T) %Then
                    %Do;
                        TRUNCATE TABLE &vTabName.
                    %End;
                %Else
                    %Do;
                        %If (&vObjectType eq S) %Then
                            %Do;
                                ALTER TABLE &vTabName. TRUNCATE SUBPARTITION &vObjectName
                            %End;
                        %Else
                            %Do;
                                %If (&vObjectType eq P) %Then
                                    %Do;
                                        ALTER TABLE &vTabName. TRUNCATE PARTITION &vObjectName
                                    %End;
                                %Else
                                    %Do;
                                        PUT ERROR: NO TRUNCATION!!! PLEASE CHECK PARAMETERS;
                                    %End;
                            %End;
                    %End;
                )BY &dbtype.;
            %RDBMS_Report(macro_call=DFR_IMPORT_P_TRUNC_TABLE,code_area=1,switch=ON);
            DISCONNECT FROM &dbtype.;
        QUIT;
%Mend;

/*------------------------------------------------------------------------------
    Demand Forecasting for Retail
    Description: This is the driver macro that kicks off importing final
                 forecasting data back into SAS Merchandise Planning systems
    Developer: Paul Dettorre, X8265
    Created: 11/2008
    Modification Log:

    Called from macro: MM_FORECAST

    Dependant Global Variables
    --------------------------------------------------------------------------------
    Load_Data_Remotely      : a value of TRUE triggers loading per-aggregated data
                              to the target RDBMS partitioned tables(s) using
                              MPCONNECT to submit multiple jobs in parallel.
    Projid                  : Project ID, a.k.a worksheet id
    SRCTAB                  : Source Table prefix
    FROM_TIME               : Physical Date/time value ????
    TO_TIME                 : Physical Date/time value ????
    INCL_RECONSTAT          :
    RECON_SINGLE            : Triggers to aggregate at lowest level only
    DFR_WF_RECON            :
    RECON_HIER              : 1 = aggregate across merchandise 2 = location
    MERCH_FROM_LEVEL        : Highest level in merchandise hierarchy to grab from
    MERCH_TO_LEVEL          : Relative position of lowest level to grab data from.
                              Example: If using merch levels 3,4,5,6,10 then this
                              would be set to 7 not 10.
    LOC_FROM_LEVEL          : Highest level in location hierarchy to grab from
    LOC_TO_LEVEL            : Lowest level in location hierarchy to grab from
    ORIG_LOC_FROM_LEVEL     :
    TIME_FROM_LEVEL         : Highest level in time hierarchy to grab from
    TIME_TO_LEVEL           : Lowest level in time hierarchy to grab from
    DFR_CLSTR_ST_ID         :
    ADJUSTEDMLEVEL          : Same relative value as MERCH_TO_LEVEL
    RECON_SINGLE            :
    DFR_WF_RECON            :
    RECON_MERCH             :
    FACT_TYPE               : F = forecast version, S = Standard Fact Table
    DFR_IMPORT_VERSION      :
    DFR_FCAST_IMPORT_KPIS   :
                              Example: mfact_version.fc_sales_r,mfact_version.fc_sales_u




------------------------------------------------------------------------------*/
%MACRO DFR_IMPORT();
    %let Load_Data_Remotely=FALSE;  /* TRUE to run import remote parellel processes */
    %if (&ma_max_rsubmit.>1) %then %do;
        %let Load_Data_Remotely=TRUE;
    %end;

    %let srctab=rm_f_&projid.;

/*  %if &merch_to_level.>&dfr_maxmerch %then %do; %let adjustedmlevel =%eval(&merch_to_level.-(9-&dfr_maxmerch.)); %end; %else %do; %let adjustedmlevel = &merch_to_level.; %end;*/

    /*
        Global Variable defining parameters needed for Oracle side of import and aggregation

        TMP_TBL_NAME          : Temp table loaded into oracle after all levels have been combined
                                but before aggregations.
        MMX_TBL_NAME          : RDBMS FACT table containing the final results
        FACT_TYPE             :
        FCAST_VERSION         :
        AGG_TO_TOP_MERCH_LEVEL:
        AGG_FROM_MERCH_LEVEL  :
        AGG_TO_TOP_LOC_LEVEL  :
        AGG_FROM_LOC_LEVEL    :
        AGG_TO_TOP_TIME_LEVEL :
        AGG_FROM_TIME_LEVEL   :
        AGG_MERCH             :
        AGG_LOC               :
        AGG_TIME              :
        PRESERVE_ML           :
        PRESERVE_TIME         :
        MERCH_TAB_NAME        :
        LOC_TAB_NAME          :
    */
    %GLOBAL TMP_TBL_NAME            MMX_TBL_NAME            FACT_TYPE               FCAST_VERSION
            AGG_TO_TOP_MERCH_LEVEL  AGG_FROM_MERCH_LEVEL    AGG_TO_TOP_LOC_LEVEL    AGG_FROM_LOC_LEVEL
            AGG_TO_TOP_TIME_LEVEL   AGG_FROM_TIME_LEVEL     AGG_MERCH               AGG_LOC
            AGG_TIME                PRESERVE_ML             PRESERVE_TIME           MERCH_TAB_NAME
            LOC_TAB_NAME            CLSTR_ST_ID             STATUS                  ;

    /*  Delete output Table to be aggregated */
        /*Proc Datasets library=out;
            Delete &SRCTAB._Final;
        Quit;*/

    /*
        Recon_Hier 1 is to process Merch Reconcilliation, 2 is for Location Reconciliation
    */

    %IF (&RECON_HIER.=1) %THEN
        %DO;
            %DO Merch_Loop=&m_lev. %TO &MERCH_FROM_LEVEL %BY -1;
                %Process_level(DSN=&SRCTAB._m&Merch_Loop._l&l_lev.,
/*                               Time_to_Level=5,*/
                               time_to_level=&time_to_level,
                               MERCH_FROM_LEVEL=&MERCH_FROM_LEVEL.,
                               MERCH_TO_LEVEL=&Merch_Loop.,
                               loc_from_level=&LOC_FROM_LEVEL.,
                               loc_to_level=&l_lev.,
                               orig_loc_from_level=&orig_loc_from_level.,
                               dfr_clstr_st_id=&dfr_clstr_st_id.,
                               FV_Version_ID=&dfr_import_version.,
                            adjustedmlevel=&m_lev.);
                /* DEM 3/6/09 BUG#565954 loc_to_level to l_lev*/
                PROC APPEND BASE=out.&SRCTAB._Final
                            DATA=out.&SRCTAB._m&Merch_Loop._l&l_lev._trans_1
                     FORCE;
                RUN;
                %If &RECON_SINGLE.=1 %Then %goto break_loop;
            %END;
        %END;
    %ELSE
        %DO;
            %DO Loc_Loop=&L_LEV. %TO &LOC_FROM_LEVEL %BY -1;

                %Process_level(DSN=&SRCTAB._m&m_lev._l&Loc_Loop.,
/*                               Time_to_Level=5,*/
                               time_to_level=&time_to_level,
                               MERCH_FROM_LEVEL=&MERCH_FROM_LEVEL.,
                               MERCH_TO_LEVEL=&m_lev.,
                               loc_from_level=&LOC_FROM_LEVEL.,
                               loc_to_level=&Loc_Loop.,
                               orig_loc_from_level=&orig_loc_from_level.,
                               dfr_clstr_st_id=&dfr_clstr_st_id.,
                               FV_Version_ID=&dfr_import_version.,
                               adjustedmlevel=&m_lev.);
                PROC APPEND BASE=out.&SRCTAB._Final
                            DATA=out.&SRCTAB._m&m_lev._l&Loc_Loop._trans_1
                     FORCE;
                RUN;
                /*  If RECON_SINGLE is 1 then only process the lowest level */
                %If &RECON_SINGLE.=1 %Then %goto break_loop;
            %END;
        %END;

    %break_loop:

    /*  Parse the table prefix from the KPI select statement */
    %let MMX_TBL_NAME=%sysfunc(kupcase(%sysfunc(kscan(%quote(&dfr_fcast_import_kpis.),1))));

    %let FACT_TYPE=S;

    %if (&MMX_TBL_NAME. eq MFACT_VERSION) %then
        %do;
            %let FACT_TYPE=F;
        %end;

    /*  Need to add conditional IF clauses to acount for other FACT_TYPE values*/

    %Let MMX_TEMP_TABLE=T_TEMP_MFACT_&PROJID.;

    data _null_;
        from_year=year("&FROM_TIME."d);
        to_year=year("&TO_TIME."d);
        call symputx('FROM_YEAR',from_year,'l');
        call symputx('TO_YEAR',to_year,'l');
    run;

    proc datasets library=maxdata;
        delete &MMX_TEMP_TABLE.;
        delete T_DFR_&PROJID._MERCH_KEYS;
        delete T_DFR_&PROJID._LOC_KEYS;
    quit;

    /*  pkg_dfr.p_cr_fcast_table
        Parameters:
            vFcastTabName   : Target forecast table
            vNewTabName     : Temp Table to upload final results from import routine
            iStartYear      : Beginning year of forecast data
            iEndYear        : Ending Year of Forecast Data
            iFvVersionId    : DFR Import Version
        Flow:
            Create a temp table with partitioning by Time Levels/ID
    */

    %p_cr_final_table(Table_Type=&FACT_TYPE.,
                      vFactTabName=&MMX_TBL_NAME.,
                      vNewTabName=&MMX_TEMP_TABLE.,
                      iStartYear=&FROM_YEAR.,
                      iEndYear=&TO_YEAR.,
                      iFvVersionId=&dfr_import_version.);

    /*  Parallel load of data into Oracle Partitions

        ADD PICE TO GET ONLY PARTITION IF NOT FCAST
        */

/* DEM/Paul 3/18/09 fix for S0566512
    proc sql noprint;
        connect to &dbtype. (user=maxdata orapw=&dbpassword. path=&DSN.);

        create table all_tab_subpartitions as
        select partition_name
             , subpartition_name
             , kscan(partition_name,1,'_') as Partition_Code
             , kscan(partition_name,2,'_') as Time_Level
             , kscan(partition_name,3,'_') as Time_ID
             , &dfr_import_version. as Fact_Version
          from connection to &dbtype.(select *
                                      from all_tab_subpartitions
                                     where table_name = %str(%'&MMX_TEMP_TABLE.%'))
        ;
    quit;

    proc sql;
        create table partition_list as
        select A.*
          from all_tab_subpartitions A
          JOIN (SELECT (&Time_to_Level. + 46) as Time_Level
                       ,LV&Time_to_Level.TIME_LKUP_ID as Time_ID
                  FROM maxapp.lv&Time_to_Level.time
                 WHERE LV&Time_to_Level.TIME_START_DATE >= "&from_time."d
                   AND LV&Time_to_Level.TIME_END_DATE   <= "&to_time."d) B
            ON input(A.Time_Level,8.) = B.Time_Level
           AND input(A.Time_ID,8.) = B.Time_ID
           AND A.Fact_Version=&dfr_import_version.;
    QUIT;
*/

    %If (&FACT_TYPE. eq F) %Then
        %Do;
            proc sql noprint;
                connect to &dbtype. (user=maxdata orapw=&dbpassword. path=&DSN.);
                create table all_tab_subpartitions as
                select partition_name
                     , subpartition_name
                     , kscan(partition_name,1,'_') as Partition_Code
                     , kscan(partition_name,2,'_') as Time_Level
                     , kscan(partition_name,3,'_') as Time_ID
                     , &dfr_import_version. as Fact_Version
                  from connection to &dbtype.(select *
                                              from all_tab_subpartitions
                                             where table_name = %str(%'&MMX_TEMP_TABLE.%'))
                ;
            quit;

            proc sql;
                create table partition_list as
                select A.*
                  from all_tab_subpartitions A
                  JOIN (SELECT (&Time_to_Level. + 46) as Time_Level
                               ,LV&Time_to_Level.TIME_LKUP_ID as Time_ID
                          FROM maxapp.lv&Time_to_Level.time
                         WHERE LV&Time_to_Level.TIME_START_DATE >= "&from_time."d
                           AND LV&Time_to_Level.TIME_END_DATE   <= "&to_time."d) B
                    ON input(A.Time_Level,16.) = B.Time_Level
                   AND input(A.Time_ID,16.) = B.Time_ID
                   AND A.Fact_Version=&dfr_import_version.;
            Quit;
        %End;
    %Else
        %Do;
            proc sql noprint;
                connect to &dbtype. (user=maxdata orapw=&dbpassword. path=&DSN.);
                create table all_tab_partitions as
                select partition_name
                     , kscan(partition_name,1,'_') as Partition_Code
                     , kscan(partition_name,2,'_') as Time_Level
                     , kscan(partition_name,3,'_') as Time_ID
                     , &dfr_import_version. as Fact_Version
                  from connection to &dbtype.(select *
                                              from all_tab_partitions
                                             where table_name = %str(%'&MMX_TEMP_TABLE.%'))
                ;
            quit;

            proc sql;
                create table partition_list as
                select A.*
                  from all_tab_partitions A
                  JOIN (SELECT (&Time_to_Level. + 46) as Time_Level
                               ,LV&Time_to_Level.TIME_LKUP_ID as Time_ID
                          FROM maxapp.lv&Time_to_Level.time
                         WHERE LV&Time_to_Level.TIME_START_DATE >= "&from_time."d
                           AND LV&Time_to_Level.TIME_END_DATE   <= "&to_time."d) B
                    ON input(A.Time_Level,16.) = B.Time_Level
                   AND input(A.Time_ID,16.) = B.Time_ID
                   AND A.Fact_Version=&dfr_import_version.;
            Quit;
        %End;
/* end fix: DEM/Paul 3/18/09 fix for S0566512 */

    %Let loadctr=1;
    %Let waitforlist=;
    %let pt_rc=%sysfunc(open(partition_list,i));
    %let fetch_part=%sysfunc(fetch(&pt_rc.));


    /*
        BAD for the bad file
        CTL for the control file
        DAT for the data file
        DSC for the discard file
        LOG for the log file
    */

    %If (&SYSSCP. eq WIN) %Then
        %Do;
            options noxwait;
            %SYSEXEC DEL &PATH_STAGE.\*.BAD;
            %SYSEXEC DEL &PATH_STAGE.\*.CTL;
            %SYSEXEC DEL &PATH_STAGE.\*.DAT;
            %SYSEXEC DEL &PATH_STAGE.\*.DSC;
            %SYSEXEC DEL &PATH_STAGE.\*.LOG;
        %End;

    data out.rm_f_&Projid._final;
        set out.rm_f_&Projid._final;
        %let Loop_Cnt=2;
        %do %while(%scan(&dfr_fcast_import_kpis.,&Loop_Cnt.) ne );
            %let NextKPI=%scan(&dfr_fcast_import_kpis.,&Loop_Cnt.);
            Format &NextKPI. 18.5;
            if (&NextKPI. gt 999999999999.0) then &NextKPI. = 999999999999.0;
			%if &dfr_wf_revisit. =1 %then %do;
	            %let NextSTD=%scan(&dfr_fcast_import_kpis_std.,&Loop_Cnt.);
	            Format &NextSTD. 18.5;
	            if (&NextSTD. gt 999999999999.0) then &NextSTD. = 999999999999.0;
			%end;
            %let Loop_Cnt=%Eval(&Loop_Cnt. + 2);
        %End;
    run;

    %do %while(&fetch_part eq 0);
        %let PartitionName=%sysfunc(getvarc(&pt_rc.,%sysfunc(varnum(&pt_rc.,partition_name)))) ;
        %let SubPartitionName=%sysfunc(getvarc(&pt_rc.,%sysfunc(varnum(&pt_rc.,subpartition_name)))) ;
        %let Partition_Code=%sysfunc(getvarc(&pt_rc.,%sysfunc(varnum(&pt_rc.,Partition_Code)))) ;
        %let Time_Level_tmp=%sysfunc(getvarc(&pt_rc.,%sysfunc(varnum(&pt_rc.,Time_Level)))) ;
        %let Time_ID_tmp=%sysfunc(getvarc(&pt_rc.,%sysfunc(varnum(&pt_rc.,Time_ID)))) ;
        %let Fact_Version_tmp=%sysfunc(getvarn(&pt_rc.,%sysfunc(varnum(&pt_rc.,Fact_Version)))) ;

        %If (&Load_Data_Remotely. eq TRUE ) %Then
            %Do;
                signon task&loadctr. sascmd='!sascmd';

                %syslput MMX_TEMP_TABLE=&MMX_TEMP_TABLE./REMOTE=task&loadctr.;
                %syslput projid=&projid./REMOTE=task&loadctr.;
                %syslput PartitionName=&PartitionName./REMOTE=task&loadctr.;
                %syslput SubPartitionName=&SubPartitionName./REMOTE=task&loadctr.;
                %syslput Time_Level_tmp=&Time_Level_tmp./REMOTE=task&loadctr.;
                %syslput Time_ID_tmp=&Time_ID_tmp./REMOTE=task&loadctr.;
                %syslput Fact_Version_tmp=&Fact_Version_tmp./REMOTE=task&loadctr.;
                %syslput dsn=&dsn./REMOTE=task&loadctr.;
                %syslput dbtype_remote=&dbtype.;
                %syslput dbpassword_remote=&dbpassword.;
                %syslput loadctr=&loadctr./REMOTE=task&loadctr.;
                %syslput PATH_STAGE=&PATH_STAGE./REMOTE=task&loadctr.;

                %if (&Time_ID_tmp. ne DEFAULT) %Then
                    %do;
                        rsubmit process=task&loadctr. wait=no Persist=YES inheritlib=(work=r_work out) LOG=KEEP;
                            options mprint symbolgen mprintnest mlogic source source2;
                            libname MAXDATA &dbtype_remote. user=maxdata orapw=&dbpassword_remote. path=&DSN.;
                            proc append base=MAXDATA.&MMX_TEMP_TABLE.(BL_BADFILE    ="&PATH_STAGE./&PartitionName._&loadctr..BAD"
                                                                      BL_CONTROL    ="&PATH_STAGE./&PartitionName._&loadctr..CTL"
                                                                      BL_DATAFILE   ="&PATH_STAGE./&PartitionName._&loadctr..DAT"
                                                                      BL_DISCARDFILE="&PATH_STAGE./&PartitionName._&loadctr..DSC"
                                                                      BL_LOG        ="&PATH_STAGE./&PartitionName._&loadctr..LOG"
                                                                      OR_PARTITION  =&PartitionName.
                                                                      BULKLOAD      =YES
                                                                      BL_LOAD_METHOD=TRUNCATE
                                                                      BL_DELETE_DATAFILE=YES)
                                        data=out.rm_f_&Projid._final
                                        FORCE;
                                        WHERE Time_Level=&Time_Level_tmp. and Time_ID=&Time_ID_Tmp. and FV_Version_ID=&Fact_Version_tmp.;
                            run;
                            libname maxdata clear;
                        endrsubmit;
                        %let waitforlist=&waitforlist. task&loadctr.;
                    %end;
            %End;
        %else
            %Do;
                proc append base=MAXDATA.&MMX_TEMP_TABLE.(BL_BADFILE    ="&PATH_STAGE./&PartitionName._&loadctr..BAD"
                                                          BL_CONTROL    ="&PATH_STAGE./&PartitionName._&loadctr..CTL"
                                                          BL_DATAFILE   ="&PATH_STAGE./&PartitionName._&loadctr..DAT"
                                                          BL_DISCARDFILE="&PATH_STAGE./&PartitionName._&loadctr..DSC"
                                                          BL_LOG        ="&PATH_STAGE./&PartitionName._&loadctr..LOG"
                                                          OR_PARTITION  =&PartitionName.
                                                          BULKLOAD      =YES
                                                          BL_LOAD_METHOD=TRUNCATE
                                                          BL_DELETE_DATAFILE=YES)
                            data=out.rm_f_&Projid._final
                            FORCE;
                            WHERE Time_Level=&Time_Level_tmp. and Time_ID=&Time_ID_Tmp. and FV_Version_ID=&Fact_Version_tmp.;
                run;
            %End;

            %let loadctr=%Eval(&loadctr + 1);
        %let fetch_part=%sysfunc(fetch(&pt_rc.));
    %End;%* End LOOP;
    %Let Close_Rc=%sysfunc(close(&pt_rc.));

    %If ((&Load_Data_Remotely. eq TRUE ) and (&waitforlist. ne )) %Then
        %Do;
            waitfor _all_ &waitforlist;

            /*  Close all remote connections*/
            %let MaxCtr=%Eval(&loadctr - 1);
            %do tctr=1 %to &MaxCtr.;
                rget task&tctr.;
                signoff task&tctr.;
            %end;
        %End;

/*    %goto break_loop;*/

    /*  Build Control Record for aggregation phase in target RDBMS
        The control template record is used by the RDBMS packages to
        idetify the source data and provide rules for aggregation and
        storage of target data to the appropriate fact table.
    */

    %Let TMP_TBL_NAME          =&MMX_TEMP_TABLE.;
    %Let MMX_TBL_NAME          =&MMX_TBL_NAME.;
    %Let FACT_TYPE             =&fact_type.;
    %Let FCAST_VERSION         =&dfr_import_version.;
    %Let AGG_TO_TOP_MERCH_LEVEL=&merch_to_level.;
    %Let AGG_FROM_MERCH_LEVEL  =&merch_from_level.;
    %Let AGG_TO_TOP_LOC_LEVEL  =&loc_to_level.;
    %Let AGG_FROM_LOC_LEVEL    =&loc_from_level.;
    %Let AGG_TO_TOP_TIME_LEVEL =%EVAL(&time_to_level. + 46);
    %Let AGG_FROM_TIME_LEVEL   =%EVAL(&time_from_level. + 46);

    %If (&Recon_single. eq 1) %Then
        %Do;
            %Let AGG_MERCH=N;
            %Let AGG_LOC  =N;
            %Let AGG_TIME =N;
        %end;
    %Else
        %Do;
            %If (&Recon_single. ne 1) and (&RECON_HIER. eq 1) %Then
                %Do;
                    %Let AGG_MERCH=N;
                    %Let AGG_LOC  =Y;
                    %Let AGG_TIME =Y;
                %end;
            %Else
                %Do;
                    %If (&Recon_single. ne 1) and (&RECON_HIER. eq 2) %Then
                        %Do;
                            %Let AGG_MERCH=Y;
                            %Let AGG_LOC  =N;
                            %Let AGG_TIME =Y;
                        %end;
                    %Else
                        %Do;
                            %Let AGG_MERCH=N;
                            %Let AGG_LOC  =N;
                            %Let AGG_TIME =Y;
                        %End;
                %end;
        %End;

    %Let PRESERVE_ML           =Y;
    %Let PRESERVE_TIME         =Y;
    %Let MERCH_TAB_NAME        =T_DFR_&PROJID._MERCH_KEYS;
    %Let LOC_TAB_NAME          =T_DFR_&PROJID._LOC_KEYS;
    %Let CLSTR_ST_ID           =&dfr_clstr_st_id.;
    %Let STATUS                =P;

    /*
        Below is code to verify and fill in missing keys
        PADETT: The exact reason for this code is not know, nor was
                I able to find out the exact reason for it, the original bug that
                caused it or how to replicate the bug.
    */

    %dfr_import_add_parents();

    proc sql noprint;
        create table work.newmerch as
        select distinct merch_id, merch_level
          from out.rm_f_&Projid._final
      order by merch_id, merch_level
        ;
    quit;

    proc sort data=stage.mkeys_&projid. out=work.mkeys;
        by merch_id merch_level;
    run;

    data out.T_DFR_&PROJID._MERCH_KEYS;
        merge work.mkeys work.newmerch;
        by merch_id merch_level;
    run;

    proc sql noprint;
        create table work.newloc as
        select distinct location_id, location_level
          from out.rm_f_&Projid._final
      order by location_id, location_level
        ;
    quit;

    proc sort data=stage.lkeys_&projid. out=work.lkeys;
        by location_id location_level;
    run;

    data out.T_DFR_&PROJID._LOC_KEYS;
        merge work.lkeys work.newloc;
        by location_id location_level;
    run;

    /*
        The following code performs the aggregation and loads the final fact table
    */
    %p_agg(vNewTabName=&MMX_TEMP_TABLE.,
             TMP_TBL_NAME=&TMP_TBL_NAME.,
             MMX_TBL_NAME=&MMX_TBL_NAME.,
             FACT_TYPE=&FACT_TYPE.,
             FCAST_VERSION=&FCAST_VERSION.,
             AGG_TO_TOP_MERCH_LEVEL=&AGG_TO_TOP_MERCH_LEVEL.,
             AGG_FROM_MERCH_LEVEL=&AGG_FROM_MERCH_LEVEL.,
             AGG_TO_TOP_LOC_LEVEL=&AGG_TO_TOP_LOC_LEVEL.,
             AGG_FROM_LOC_LEVEL=&AGG_FROM_LOC_LEVEL.,
             AGG_TO_TOP_TIME_LEVEL=&AGG_TO_TOP_TIME_LEVEL.,
             AGG_FROM_TIME_LEVEL=&AGG_FROM_TIME_LEVEL.,
             AGG_MERCH=&AGG_MERCH.,
             AGG_LOC=&AGG_LOC.,
             AGG_TIME=&AGG_TIME.,
             PRESERVE_ML=&PRESERVE_ML.,
             PRESERVE_TIME=&PRESERVE_TIME.,
             MERCH_TAB_NAME=&MERCH_TAB_NAME.,
             LOC_TAB_NAME=&LOC_TAB_NAME.,
             CLSTR_ST_ID=&CLSTR_ST_ID.,
             STATUS=&STATUS.);

    /* To delete temporary datasets in OUT library. ( _TEMP001, _TRANS, _TRANS_1) */
       %deltempds_outlib;


    %goto normal_termination;


    %normal_termination:

%MEND;



