/**************************************************************/
/*                                                            */
/*       SAS RETAIL INTELLIGENCE SOLUTIONS                    */
/*       Demand Forecast For Retail Planning 4.2              */
/*                                                            */
/**************************************************************/
/*                                                            */
/* SAS Retail Group, All rights reserved (c) 2006,2007,2008   */
/*                                                            */
/**************************************************************/

/****************************************************************/
/* FUNCTIONALITY :- THE MAIN FORECASTING ROUTINE FOR DFR		*/
/****************************************************************/
 
%macro mm_forecast(wktid=,dsn=,dbtype=,standalone=0,paramfile=,set_analytic=0);
 
	/*********************************/
	/*  WORKFLOW                     */
	/*********************************/
/*
Standalone :-
	=0 ATTACHED TO DATABASE   Only Extract and Import require attachment to database
	=1 Standalone Not attached to Database
 
DFR_SETUP :-	Setup metadata, dimension levels, reconciliation parameters
DFR_EXTRACT :- 	Extract history including new item history, Aggregate locations, 
				add full time weeks, filter time-series, create input forecast dataset
	EXTRACT ONLY PERFORMED IF NOT STANDALONE, ASSUMPTION IS FIRST RUN IS CONNECTED, STANDALONE=0
Data set out:   stage.f_[projid]
EXECUTE FORECAST    Diagnose, Select, Fit, Forecast, Revisit, Reconciliation
Data set in:    inp.f_[projid]
Data set out:   stage.re_f_[projid]_m[merch lev]_l[loc level] reconciled or final forecast by level (consolidated over KPIs)
stage.outsum_f_[projid]_m[merch lev]_l[loc level]_KPI       forecast statistics by level and KPI
stage.outstat_f_[projid]_m[merch lev]_l[loc level]_KPI      forecast statistics by level and KPI
for.urm_f_[projid]_m[merch lev]_l[loc level]                unreconciled forecast [not use in import]
 
DFR_EXCEPTIONS :-	Produce exception dataset based on filter for analysis in SAS Forecast Studio or other tool
Data set out:   out.fe_[jobname]_[projid]

DFR_IMPORT :-	Import forecast into merchandise planning
	IMPORT ONLY PERFORMED IF NOT STANDALONE (ie, MUST BE CONNECTED TO DATABASE)
	Data set in :    stage.re_f_[projid]_m[merch lev]_l[loc level]
	Output :    mfact_version merchandise planning table
 
MM_ARCHIVE :- 	Archive the rm (forecast), outstat, and outsum datasets to archive folder
	Output: ARCHIVE_##_[projid]_[ArchiveDate YYYYMMDD]      
	archive folder name where ## is the archive folder number
DFR_PURGE :-	Purge working data sets

*/
 
/********************************************/
/*********** SETUP **************************/
/********************************************/
 
%if (&set_analytic.>0)
	%then %let analytic_id=&set_analytic.;       /* function passed in as argument */
	%else %let analytic_id=&analytic_lkup_id.;   /* from process server */

/* Read and set dfr parameters from jobparam_[projid] dataset */
/* Also initializes ods report */
%if &standalone.<2 %then %do;
	%initialsetup(proj=&wktid.,type=%quote('ma','dfr'), standalone=&standalone., paramfile=&paramfile.);
%end;
 

/* Setup metadata, dimension levels, reconciliation parameters */
/*
%include "&path_pgm.&slash.dfrcode&slash.dfr_utils.sas";
%include "&path_pgm.&slash.dfrcode&slash.dfr_setup.sas"; */

%if (&job_rc.<5) %then %do;
	%dfr_setup;
	%timecheck(step=%quote(SETUP));
%end;

/********************************************************************************************/
/********************************************************************************************/
/************************************** DATA EXTRACT ****************************************/
/* Add Secondary Effects to the dataset f_[projid] ******************************************/
/********************************************************************************************/
/********************************************************************************************/ 

%if (&job_rc.<5) %then %do;
	%ABT_Creation;
%end;
/* EXIT if only performing history imports for new items */
%if ((&dfr_wf_newitem.=3) or (&dfr_wf_newitem.=4)) %then %do;
	%goto close_out;
%end;

%if (&dfr_wf_execmethod.>0) and (&job_rc.<5) %then %do;
		/********************************************************************************************/
		/********************************************************************************************/
		/**** Splitting datasets ********************************************************************/
		/********************************************************************************************/
		/********************************************************************************************/

		%if (&job_rc.<5) %then %do;
			%Splitter;
		%end;
		/********************************************************************************************/
		/********************************************************************************************/
		/**** Forecast Execution ********************************************************************/
		/********************************************************************************************/
		/********************************************************************************************/
		%if (&job_rc.<5) %then %do;
			%hpf_call;
		%end;

		/********************************************************************************************/
		/********************************************************************************************/
		/**** reconcile and merge output ************************************************************/
		/********************************************************************************************/
		/********************************************************************************************/
		%if (&job_rc.<5) %then %do;
			%Reconcile_merge;
		%end;

%end; /* FORECAST EXECUTION */

/********************************************************************************************/
/********************************************************************************************/
/* IMPORT forecast back into SAS Merchandise Planning ***************************************/
/********************************************************************************************/
/********************************************************************************************/

%if (&job_rc.<5) %then %do;
		/* Reset to levels to original */

	%let merch_to_level=&fcast_merch_to_level.;
	%let loc_from_level=&fcast_loc_from_level.;
	%let loc_to_level=&fcast_loc_to_level.;
	%mm_step_finish(proj=&projid.,step=hpf,standalone=&standalone.);

	data mmdata.Jobparam_&projid.;
		set mmdata.Jobparam_&projid._copy;
	run;
	proc sql noprint;
		drop table mmdata.Jobparam_&projid._copy;
	quit;

	%if (&dfr_wf_import.=1) and (&job_rc.<5) and (&standalone ^= 1) %then %do;
		%dfr_import;
		%mm_step_finish(proj=&projid.,step=import,standalone=&standalone.);	
		%timecheck(step=%quote(IMPORT));
	%end;
%end;
 
/********************************************************************************************/
/********************************************************************************************/
/* 							ARCHIVE                                                         */
/********************************************************************************************/
/********************************************************************************************/
%if (&dfr_wf_archive.=1) and (&dfr_archive_count.>0) and (&job_rc.<5) %then %do;
	%mm_archive(
		mm_archive_backup_root=&path_archive.,
		mm_archive_ngen=&dfr_archive_count.,
		mm_archive_wsid=&projid.,
		mm_libs=for out,
		mm_files=OUTSTAT_F_&projid._ OUTSUM_F_&projid._ RM_F_&projid._
		);
	 
	%mm_step_finish(proj=&projid.,step=archive,standalone=&standalone.);
	%timecheck(step=%quote(ARCHIVE));
%end;
 
%put ARCHIVE &job_rc;
 
/********************************************************************************************/
/********************************************************************************************/
/* 							CLOSE PROCESS                                                   */
/********************************************************************************************/
/********************************************************************************************/
%close_out:
 
%if (&dfr_wf_purge.=1) and (&job_rc.<5) %then %do;
	%dfr_purge;

	/*Code to delete intermediate datasets(WORK)*/
	proc datasets library=W_&wktid. kill nodetails nolist;
	quit;

	%timecheck(step=%quote(PURGE));
%end;
 
%mm_endprocess(standalone=&standalone);
%if (&dfr_wf_newitem.<4) and (&dfr_wf_execmethod.>0) %then %do;  /* forecast process ran so update report */
	/*%ID_2_Name(html_file=&rptout);*/
%end;
 
%mend mm_forecast;
