 %*+----------------------------------------------------------------+
   | Copyright (c) 2006 by SAS Institute Inc., Cary, NC 27513-8000
   | Name:     smd2ds.sas
   | Product:  SAS Base - NLS
   | Language: SAS macro
   | Purpose:  Create a sas message data set dataset from .smd files
               for the SASMSG function.
   | Author:   FRAMBO - Mickael Bouedo
   | Support:  FRAMBO - Mickael Bouedo
   |
   | Usage:    %SMD2DS(dir=,basename=,locale=,lib=)

   | Parameters:
   |  dir=       Requiered. The name of the directory where are
   |             located the .smd files to process.
   |
   |  basename=  Required. The name of the file whithout locale
   |             attribute.
   |             i.e:  myfic  i the basename of myfic_fr_FR.smd
   |
   |  locale=    Optional. List of locale, separated by a blank,
   |             to include in addition to the default file.
   |
   |  lib=       Optional. The name of the library where to create
   |             the dataset. Default is the WORK library.
   |
   |  DIR and BASENAME are required.
   |
   | Output:   A SAS dataset for the SASMSG function.
   |
   | History:  who    date    action
   |           frambo 28sep06 Creation.
   |           frambo 20nov06 S0391498: Fixed warning, upcase key
   |           frambo 27nov06 S0392614: Delete temporary data set
   | Notes:
   | end
   +----------------------------------------------------------------+;


%MACRO smd2ds(DIR=,BASENAME=,LOCALE=,LIB=) ;

  %LOCAL SEP TABLE SIZE TMPAF TMPAK ENCODING ECODE;
  %LET ECODE = 0 ;
  %LET SIZE=1200;
  %LET TMPAF=__smd_all_files ;
  %LET TMPAK=__smd_keys_values ;
  %LET ENCODING=asciiany ;

  %LET DIR=%quote(&DIR);


  %IF (&SYSSCP EQ OS) %THEN %DO ;
      %PUT ERROR: The %SMD2DS macro is not supported on this Operating System. ;
      %GOTO EXIT2 ;
  %END;

  %IF (&BASENAME EQ ) %THEN %DO ;
     %PUT  ERROR: The parameter BASENAME is required. ;
     %GOTO EXIT2 ;
  %END ;

  %IF ( (&SYSSCP=WIN) OR  (&SYSSCP=DNTHOST) ) %THEN
     %LET SEP =\;
  %ELSE %LET SEP=/;


  %IF (&LIB  EQ ) %THEN %LET LIB=WORK ;

  %LET TABLE=&LIB..&BASENAME ;

  %IF (&DIR  EQ ) %THEN %DO ;
     %PUT  ERROR: The parameter DIR is required. ;
     %GOTO EXIT2 ;
  %END ;


  %IF (&LOCALE NE ) %THEN %DO ;
     data _null_ ;
        length locale $ 5 filedir $ 200 ;
        x = symget('LOCALE');
        y = strip(symget('DIR')) ;
        b = strip(symget('BASENAME')) ;
        s = symget('SEP') ;
        n = count(strip(x),' ') + 1 ;
        locale = "en" ;
        filedir = strip(y) || strip(s) || strip(b) || ".smd" ;
        if (fileexist(filedir)=0) then do ;
           fic_en = strip("&DIR") || strip("&SEP") || strip("&BASENAME") || "_en.smd" ;
           if (fileexist(fic_en)=0) then do ;
              putlog "ERROR: Default English file does not exist." ;
              fic = strip("&BASENAME") || ".smd" ;
              fic_en = strip("&BASENAME") || "_en.smd" ;
              errmsg = 'ERROR: The macro did not find one of these files: "'
                      || strip(fic) || '", "' || strip(fic_en) || '".' ;
              putlog errmsg ;
              call symput('ecode',put(1,8.)) ;
           end ;
           else filedir = strip(fic_en) ;
        end ;
        call symput(compress('LOC0'),strip(locale)) ;
        call symput(compress('filedir0'),strip(filedir)) ;

        k=0 ;
        do i=1 to n ;
           locale = kscan(strip(x),i) ;
           filedir = strip(y) || strip(s) || strip(b) || "_" || strip(locale) || ".smd" ;
           if ((locale ne "") and (upcase(locale) ne "EN") ) then do ;
              if (fileexist(filedir)=0) then do ;
                 putlog "ERROR: The file " filedir " does not exist." ;
                 call symput('ecode',put(1,8.)) ;
              end ;
              else do ;
                 k=k+1 ;
                 call symput(compress('LOC'||put(k,3.)),strip(locale)) ;
                 call symput(compress('filedir'||put(k,3.)),strip(filedir)) ;
              end ;
           end ;
        end ;
        call symput('nblocale',put(k,8.)) ;
     run ;

     %IF (&ECODE) %THEN
        %GOTO EXIT2 ;

     data &TMPAF ;
        length locale $ 5 fic $ 200 ;
     %DO i = 0 %TO &NBLOCALE ;
        locale = "&&LOC&i" ;
        fic    = "&&FILEDIR&i" ;
        output ;
     %END ;
     run ;
  %END ;
  %ELSE %DO ;
     data &TMPAF ;
        keep locale fic ;
        length locale $ 5 fic $ 200 ;
        locale = "en" ;
        fic = strip("&DIR") || strip("&SEP") || strip("&BASENAME") || ".smd" ;
        if (fileexist(fic)=0) then do ;
           fic_en = strip("&DIR") || strip("&SEP") || strip("&BASENAME") || "_en.smd" ;
           if (fileexist(fic_en)=0) then do ;
              putlog "ERROR: Default English file does not exist." ;
              fic = strip("&BASENAME") || ".smd" ;
              fic_en = strip("&BASENAME") || "_en.smd" ;
              errmsg = 'ERROR: The macro did not find one of these files: "'
                      || strip(fic) || '", "' || strip(fic_en) || '".' ;
              putlog errmsg ;
              call symput('ecode',put(1,8.)) ;
           end ;
           else fic = strip(fic_en) ;
        end ;
     run ;

     %IF (&ECODE) %THEN
        %GOTO EXIT1 ;
  %END ;


  %IF %SYSFUNC(exist(&TABLE)) %THEN %DO ;
     proc datasets lib=&LIB memtype=data ;
        delete &BASENAME ;
     run ;
     quit ;
  %END ;


   data &TMPAK (compress=yes) ;
   set &TMPAF ;
      infile dummy filevar=fic end=done LRECL=10000 PAD ;
      do until(done) ;
        input line $10000. ;
        output ;
      end ;
   run ;

   data &TABLE (encoding=&ENCODING compress=yes) ;
      set &TMPAK ;
      length line25 $ 25 ;
      keep locale key lineno text ;
      attrib locale length=$ 5 ;
/*I18N-scnhuh-20080506-1653*/
/*change from $ 100 to $99, otherwise function sasmsg() does not work*/      
      attrib key    length=$ 60 ;
/*I18N-scnhuh-20080506-1653*/      
      attrib lineno length=5   ;
      attrib text   length=$ &SIZE ;
      retain lineno 1 PerlExpComment PerlExpKey;
      if _N_ = 1 then do ;
         comment = "/^\s*(#|\/\*|\*)/";
         validkey = "/=/" ;
         PerlExpComment = prxparse(comment);
         PerlExpKey     = prxparse(validkey);
         if missing(PerlExpComment) then do;
            putlog "ERROR: Invalid regexp " comment;
            stop;
         end;
         if missing(PerlExpKey) then do;
            putlog "ERROR: Invalid regexp " validkey ;
            stop;
         end;
      end;

      rc = prxmatch(PerlExpComment, line) ;
      if ( rc>0 or strip(line)=' ') then delete ;

      re = prxmatch(PerlExpKey, line) ;
      if (re > 0) then do ;
         key = upcase(substr(line,1,re-1)) ;
         value = substr(line,re+1) ;
      end ;
      else do ;
         if (length(line)>25) then
            line25 = substr(strip(line),1,22)||"..." ;
         else line25 = strip(line) ;
         putlog "ERROR: Invalid key\value pair: " line25 ;
         delete ;
      end ;

      v = strip(value) ;
      l = length(v) ;
      n = int(l/&SIZE) ;
      m = mod(l,&SIZE) ;
      if (m > 0) then n = n + 1 ;

      if (l>&SIZE) then do ;
         do i=n-1 to 1 by -1;
            lineno = i+1 ;
            text = substr(v,1,&SIZE) ;
            v = substr(v,&SIZE+1) ;
            output ;
         end ;
         text = v ;
         lineno = i+1 ;
         output ;
      end ;
      else do ;
         text = strip(v) ;
         output ;
      end ;
   run ;


   proc sort data=&TABLE nodupkey ;
     by locale key descending lineno;
   run ;


%EXIT1:
   proc datasets lib=&LIB memtype=data nolist ;
   %IF %SYSFUNC(exist(&TABLE)) %THEN %DO ;
     modify &BASENAME  ;
     index create indx=(LOCALE KEY) ;
   %END ;
   run ;
   quit ;

   proc datasets lib=WORK memtype=data nolist nodetails ;
   %IF %SYSFUNC(exist(&TMPAK)) %THEN %DO ;
     delete &TMPAK ;
   %END ;
   %IF %SYSFUNC(exist(&TMPAF)) %THEN %DO ;
     delete &TMPAF ;
   %END ;
   run ;
   quit ;

%EXIT2:
%MEND smd2ds ;
